#lang racket 
(require racket/runtime-path 
         (only-in "fft2.rkt" init-d) 
         "fft0.rkt" 
         "fft1.rkt" 
         "fft5.rkt")
(provide fft-in-c 
         run-fft-in-c 
         run-fft-in-c/d) 

(define-runtime-path code "in-c/fft.c")
(define-runtime-path output "in-c/output.txt")
(define-runtime-path args-file "in-c/args.h")
(define-runtime-path input-data-file "in-c/input-data.txt")
(define-runtime-path exec "in-c/fft")

;;fft-in-c : (listof float complex) -> (listof float complex)
(define (fft-in-c l) 
  (fft-in-c/private l #t #f (init-d)))

(define (fft-in-c/private l compile-code? is-benchmark? d) 
  (when compile-code? 
    (build-code-with-input l is-benchmark? d)) 
  (get-program-output exec is-benchmark?))
  
;;run-fft-in-c : (listof float complex) -> (values (listof float complex) double double double)
(define (run-fft-in-c l) 
  (run-fft-in-c/d l (init-d)))

(define (run-fft-in-c/d l d) 
  (define real-time (fft-in-c/private l #t #t d)) 
  ;Return dummy values for other fields (all we care about in benchmarking 
  ;C is real time) 
  (values '() 0 real-time 0))
  
  
;;get-program-output : path -> any
(define (get-program-output expath is-benchmark?) 
  (define proc-ports (process (path->string expath))) 
  (define stdout (first proc-ports)) 
  (define stdin (second proc-ports)) 
  (define stderr (fourth proc-ports))
  ((fifth proc-ports) 'wait)
  (define exit-code ((fifth proc-ports) 'exit-code)) 
  (unless (zero? exit-code) 
    (error 'get-program-output (format "C FFT failed with exit code ~a." exit-code)))
  
  (define ret (cond 
    [is-benchmark? (read stdout)] ;'Benchmark' version just prints real time (no result values) 
    [else 
     ;-0x1.994688cec574cp-4+0x1.457054db2917p-6i
     (let loop ([line (read-line stdout)])
       (cond
         [(eof-object? line) '()]
         [else
          (define adjusted-prefix (string-append "#x" (regexp-replace* "0x" line "")))
          (define ps (regexp-match* #rx"p[-+][0-9]+" adjusted-prefix)) 
          (define realp (string->number (regexp-replace #rx"p" (first ps) ""))) 
          (define imagp (string->number (regexp-replace #rx"p" (second ps) ""))) 
          (define no-ps (regexp-replace* #rx"p[-+][0-9]+" adjusted-prefix "")) 
          
          (define n (string->number no-ps))
          (define c (make-rectangular (* (real-part n) (expt 2 realp)) 
                                      (* (imag-part n) (expt 2 imagp))))
          
          (unless n
            (error 'fft-in-c.rkt "not anumber ~s" no-ps))
          (cons c (loop (read-line stdout)))]))]))

  (close-input-port stdout) 
  (close-output-port stdin) 
  (close-input-port stderr)
  ret)

(define last-build-args #f)

;;build-code-with-input : (listof float complex) bool -> void
(define (build-code-with-input l is-benchmark? d)
  (define args (list l is-benchmark? d))
  (unless (and last-build-args (equal? args last-build-args)) 
    (define input-len (length l)) 
    (call-with-output-file args-file
      (λ (port) 
        (if is-benchmark? 
            (fprintf port "#define BENCHMARK\n") 
            (fprintf port "#define PRINT_RESULT\n"))
        (fprintf port "#define LEN ~a\n" input-len)
        (fprintf port "#define D ~a\n" d) 
        (fprintf port "#define DATA_PATH \"~a\"\n" input-data-file))
      #:exists 'replace) 
    (call-with-output-file input-data-file 
      (λ (port) 
        (for ([n (in-list l)]) 
          (fprintf port "~a|~a|\n" (real-part n) (imag-part n)))) 
      #:exists 'replace)
    ;Build 
    (unless (system (format "gcc -o ~a -O2 -lpthread -lm -DUSE_INPUT_FILE ~a" 
                            exec 
                            code))
      (error 'fft5-in-c "C code build failed."))) 
  (set! last-build-args args))








