#lang racket/base

(require "ffts.rkt"
         "fft-test-data.rkt")

(define (ensure-same line input)
  (define fft0-ans (fft0 input))

  (define (check-on fft)
    (define got (fft input))
    (set! tests (+ 1 tests))
    (unless (equal? got fft0-ans)
      (set! failures (+ 1 failures))
      (eprintf "fft-test.rkt test case for data in fft-test-data.rkt:~a failed; fft0 is not the same as ~a; ~a\n"
               line
               (object-name fft)
               (diff-explanation got fft0-ans))))
  
  (check-on fft1)
  (check-on fft2)
  (check-on fft3)
  (check-on fft4)
  (check-on fft5))

(define (diff-explanation l1 l2)
  (cond
    [(not (= (length l1) (length l2)))
     (format "different lengths; ~a vs ~a" 
             (length l1) (length l2))]
    [else
     (define l (for/list ([e1 (in-list l1)]
                          [e2 (in-list l2)])
                 (magnitude (- e1 e2))))
     (format "max delta ~a" (apply max 0 l))]))

(define tests 0)
(define failures 0)

(for ([td (in-list test-data)])
  (ensure-same (td-line td) (td-data td)))

(cond
  [(zero? failures)
   (printf "all ~a tests passed.\n" tests)]
  [else
   (eprintf "~a test~a failed (out of ~a total)\n"
            failures
            (if (= failures 1) "" "s")
            tests)])
