#lang racket/base

;; fft0
;; sequential, list-based, untyped

(provide fft0
         decimate-in-time
         twiddle-factor)

(require (only-in racket/math pi))

;; START fft0
(define (fft0 as)
  (cond 
    [(= (length as) 1) as]
    [else
     (define-values (ss ts)
       (decimate-in-time as))
     (define bs (fft0 ss))
     (define cs (twiddle-factor (fft0 ts)))
     (append (for/list ([b (in-list bs)]
                        [c (in-list cs)])
               (+ b c))
             (for/list ([b (in-list bs)]
                        [c (in-list cs)])
               (- b c)))]))

(define (decimate-in-time as)
  (for/fold ([evens '()]
             [odds  '()])
      ([i (in-naturals)]
       [a (in-list (reverse as))])
    (if (odd? i)
        (values (cons a evens) odds)
        (values evens (cons a odds)))))

(define (twiddle-factor cs)
  (define n (length cs))
  (for/list ([k (in-naturals)]
             [c (in-list cs)])
    (* c (exp (/ (* pi 0+1i k) n)))))
;; STOP fft0


(provide run-fft0)
(define (run-fft0 l)
  (collect-garbage) (collect-garbage) (collect-garbage)
  (collect-garbage) (collect-garbage) (collect-garbage)
  (time-apply fft0 (list l)))




(module+ main
  (require racket/list)
  (define res
    (fft0 (for/list ([t 1024]) (sin (* t (/ (* 2 pi) 500))))))

  (define to-plot
    (for/list ([r res]
               [i (in-naturals)])
      (vector i (real-part r))))

  (require plot)
  (plot-new-window? #t)
  (plot (points to-plot))
  (read-line))
