#lang racket/base

;; fft2
;; parallel (restricted splitting), list-based, untyped

(provide fft2 fft2/depth init-d)

(require (only-in racket/math pi)
         racket/future
         racket/fixnum
         "fft0.rkt") ; for sequential version

;; START fft2
(define (fft2 as)
  (fft2/depth as (init-d)))

(define (init-d)
  (define pc (processor-count))
  (fl->fx (ceiling (/ (log pc) (log 2)))))

(define (fft2/depth as d)
  (cond 
    [(= (length as) 1) as]
    [(= d 0) (fft0 as)]
    [else
     (define-values (ss ts)
       (decimate-in-time as))
     (define bs
       (future 
        (λ ()
          (fft2/depth ss (- d 1)))))
     (define cs 
       (future
        (λ ()
          (twiddle-factor
           (fft2/depth ts (- d 1))))))
     (append
      (for/list ([b (in-list (touch bs))]
                 [c (in-list (touch cs))])
        (+ b c))
      (for/list ([b (in-list (touch bs))]
                 [c (in-list (touch cs))])
        (- b c)))]))
;; STOP fft2

(provide run-fft2)
(define (run-fft2 l)
  (collect-garbage) (collect-garbage) (collect-garbage)
  (collect-garbage) (collect-garbage) (collect-garbage)
  (time-apply fft2 (list l)))

(provide run-fft2/d) 
(define (run-fft2/d l d) 
  (collect-garbage) (collect-garbage) (collect-garbage)
  (collect-garbage) (collect-garbage) (collect-garbage)
  (time-apply fft2/depth (list l d)))




(module+ main
  (require racket/list)
  (define res
    (fft2 (for/list ([t 1024]) (sin (* t (/ (* 2 pi) 500))))))

  (define to-plot
    (for/list ([r res]
               [i (in-naturals)])
      (vector i (real-part r))))

  (require plot)
  (plot-new-window? #t)
  (plot (points to-plot))
  (read-line))
