#lang racket
(require racket/pretty 
         racket/flonum  
         (only-in "fft5.rkt" run-fft5 run-fft5/d) 
         "fft-in-c.rkt") 

(define ffts 
  (list (cons 'fft5 run-fft5/d) 
        (cons 'fft-in-c run-fft-in-c/d)))

;Get times for each using default depth (init-d) 
(define NUM-SAMPLES 5)
(define INPUT-SIZE (expt 2 22)) ;Power of 2
(define MAX-DEPTH 8)

(define (make-data n)
  (for/list ([t n]) (+ (sin (* t (/ (* 2 pi) 500)))
                       0.0+0.0i)))

(define l (make-data INPUT-SIZE))

(require racket/runtime-path)
(define-runtime-path times-dir "./times")

(define (run iteration) 
  ;Times for increasing depth 
  (eprintf "DEPTH TIMES\n\n")
  (define depth-times 
    (for/list ([d (in-range MAX-DEPTH)]) 
      (cons d (for/list ([f (in-list ffts)]) 
                (define name (car f)) 
                (define fun (cdr f)) 
                (define times (for/list ([i (in-range 0 NUM-SAMPLES)])
                                (collect-garbage) (collect-garbage) (collect-garbage)
                                (eprintf "~a (depth ~a), sample ~a of ~a\n" name d (add1 i) NUM-SAMPLES)
                                (define-values (ret cpu real gc) 
                                  (fun l d))
                                (unless (zero? gc) 
                                  (eprintf "WARNING: Non-zero GC time (~a ms)" gc))
                                real)) 
                (cons name times)))))
  (call-with-output-file (format "~a/run~a.rkt" times-dir iteration) 
    (λ (port) 
      (define input-size-name (format "INPUT-SIZE-~a" iteration)) 
      (define output-name (format "depth-times-~a" iteration))
      (fprintf port "#lang racket\n") 
      (fprintf port ";Generated by time-ffts.rkt\n")
      (fprintf port "(provide ~a ~a)\n\n" input-size-name output-name) 
      
      (fprintf port "(define ~a ~a)\n\n" input-size-name INPUT-SIZE)
      (fprintf port "(define ~a\n" output-name) 
      (fprintf port " ") 
      (pretty-print depth-times port)
      (fprintf port ")\n\n")) 
    #:exists 'truncate))

(when (directory-exists? times-dir) 
  (delete-directory/files times-dir)) 
(make-directory times-dir)

(for ([i (in-naturals)]) 
  (eprintf "Running iteration ~a...\n" (+ i 1))
  (run i))
