#lang racket/base
(require racket/runtime-path
         racket/set
         racket/match)
(provide depth-times INPUT-SIZE number-of-runs)

(define-runtime-path times "times")

(define INPUT-SIZE (expt 2 22))

(define (to-inexacts s)
  (cond
    [(pair? s)
     (cons (to-inexacts (car s))
           (to-inexacts (cdr s)))]
    [(number? s)
     (exact->inexact s)]
    [else s]))

;; table : (cons int[depth] (or/c 'fft5 'fft-in-c)) -o> (set number)
(define table (make-hash))
(define (add-em depth em)
  (define k (list depth (car em)))
  (hash-set! table 
             k
             (append (hash-ref table k '())
                     (cdr em))))
(define (fetch-one depth id)
  (define k (list depth id))
  (cons id (sort (hash-ref table k) <)))
  
(define max-depth 0)

(for ([f (in-list (directory-list times))])
  (define (get fmt . args)
    (dynamic-require (build-path times f)
                     (string->symbol (apply format fmt args))))
  (let/ec k
    (unless (file-exists? (build-path times f))
      (k #f))
    (define m (regexp-match #rx"run([0-9]*)[.]rkt" (path->string f)))
    (unless m
      (eprintf "unknown file: ~a" f)
      (k #f))
    (define n (string->number (list-ref m 1)))
    (define f-size (get "INPUT-SIZE-~a" n))
    (unless (= f-size INPUT-SIZE)
      (eprintf "INPUT-SIZE wrong in ~a, expected ~a, got ~a" f INPUT-SIZE f-size))
    (define this-depth-times (get "depth-times-~a" n))
    (for ([line (in-list this-depth-times)])
      (match-define (list depth fft5-nums fft-in-c-nums) line)
      (set! max-depth (max max-depth depth))
      (add-em depth fft5-nums)
      (add-em depth fft-in-c-nums))))

;; depth-times :
;; (listof (list (cons number[run<n>.rkt] number[id-in-run<n>.rkt])
;;               (cons 'fft (listof number))
;;               (cons 'fft-in-c (listof number))))
;; each element of depth-times corresponds to a specific run,
;; some number of which are extracted from one of the run<n>.rkt files.
(define depth-times
  (for/list ([i (in-range (+ max-depth 1))])
    (list i 
          (fetch-one i 'fft5)
          (fetch-one i 'fft-in-c))))


(define base-runs (list 0 'fft5))
(define number-of-runs (length (hash-ref table base-runs)))
(for ([(k v) (in-hash table)])
  (unless (= (length v) number-of-runs)
    (eprintf "times.rkt: found a different number of runs for ~s than ~s; ~s vs ~s\n"
           base-runs
           k
           (set-count v) number-of-runs)))